#-----------------------------------------------------------------
# mazeroute.tcl
#-----------------------------------------------------------------
# Defines procedure "mazeroute <netlist>", requiring a .net file as
# an argument.  Attempts a single-pass maze route of the contents
# of the netlist.
#-----------------------------------------------------------------

global Opts

proc mazeroute {netfile} {
   if [catch {open  $netfile r} fnet] {
      set netname [file rootname $netfile]
      if [catch {open  ${netfile}.net r} fnet] {
         puts stderr "Can't read netlist file $netfile"
	 return 1;
      }
   }
   # 1st line of the netlist file is throw-away
   gets $fnet line

   set destnet {}
   while {[gets $fnet line] >= 0} {
      if {$line == ""} {
	 set destnet {}
      } elseif {$destnet == {}} {
	 set destnet $line
      } else {
	 set startnet $line
	 iroute route -dlabel $destnet -slabel $startnet
	 set destnet $startnet
      }
   }
}

#-----------------------------------------------------------------
# Interactive mazerouter GUI.  Shows a list of nets from the
# selected netlist.  Allows one to select the order of routing,
# route individual nets, rip up individual nets, etc.
#
# This GUI more or less replaces the "specialopen netlist" window.
#-----------------------------------------------------------------

proc genmazehelper {} {

   if {![catch {wm deiconify .mazehelper ; raise .mazehelper}]} {return}

   toplevel .mazehelper
   frame .mazehelper.mazemenu
   frame .mazehelper.unrouted
   frame .mazehelper.transfer
   frame .mazehelper.routed

   pack .mazehelper.mazemenu -side top -anchor w

   button .mazehelper.mazemenu.load -text "Load" -command {loadnetlist}
   label .mazehelper.mazemenu.netlist -text "(no netlist loaded)"

   pack .mazehelper.mazemenu.load -side left
   pack .mazehelper.mazemenu.netlist -side left

   label .mazehelper.unrouted.title -text "Unrouted:"
   listbox .mazehelper.unrouted.contents -background white

   label .mazehelper.routed.title -text "Routed:"
   listbox .mazehelper.routed.contents -background white

   button .mazehelper.transfer.ripup -text "<--" -command {ripupnet}
   button .mazehelper.transfer.route -text "-->" -command {routenet}

   pack .mazehelper.transfer.ripup -side top
   pack .mazehelper.transfer.route -side top

   pack .mazehelper.unrouted.title -side top
   pack .mazehelper.unrouted.contents -side top -fill both -expand true

   pack .mazehelper.routed.title -side top
   pack .mazehelper.routed.contents -side top -fill both -expand true

   pack .mazehelper.unrouted -side left -fill both -expand true
   pack .mazehelper.transfer -side left
   pack .mazehelper.routed -side left -fill both -expand true
}

proc loadnetlist {} {

   set netfile [ tk_getOpenFile -filetypes \
	{{NET {.net {.net}}} {"All files" {*}}}]
   
   if [catch {open  $netfile r} fnet] {
      set netname [file rootname $netfile]
      if [catch {open  ${netfile}.net r} fnet] {
         puts stderr "Can't read netlist file $netfile"
	 return 1;
      }
   }

   # Clear out the listbox contents.
   .mazehelper.unrouted.contents delete 0 end
   .mazehelper.routed.contents delete 0 end


   # 1st line of the netlist file is throw-away
   gets $fnet line

   set currentnet {}
   while {[gets $fnet line] >= 0} {
      if {$line == ""} {
	 if {[llength $currentnet] > 0} {
	    .mazehelper.unrouted.contents insert end $currentnet
            set currentnet {}
	 }
      } else {
	 lappend currentnet $line
      }
   }

   .mazehelper.mazemenu.netlist configure -text $netfile

   close $fnet
}

#-----------------------------------------------------------------
# Get the selected network and maze route it
#-----------------------------------------------------------------

proc routenet {} {
   set cidx [.mazehelper.unrouted.contents curselection]
   set rlist [.mazehelper.unrouted.contents get $cidx]
   for {set i 1} {$i < [llength $rlist]} {incr i} {
      set j $i
      incr j -1
      set startnet [lindex $rlist $j]
      set destnet [lindex $rlist $i]
      iroute route -slabel $startnet -dlabel $destnet
   }

   .mazehelper.unrouted.contents delete $cidx $cidx
   .mazehelper.routed.contents insert end $rlist
   .mazehelper.unrouted.contents selection set $cidx
}

#-----------------------------------------------------------------
# Get the selected network and remove it
#-----------------------------------------------------------------

proc ripupnet {} {
   set cidx [.mazehelper.routed.contents curselection]
   set rlist [.mazehelper.routed.contents get $cidx]
   set netname [lindex $rlist 0]
   set layertype [goto $netname]
   select more box $layertype	;# chunk
   select more box $layertype	;# region
   select more box $layertype	;# net
   delete

   .mazehelper.routed.contents delete $cidx $cidx
   .mazehelper.unrouted.contents insert end $rlist
   .mazehelper.routed.contents selection set $cidx
}

#-----------------------------------------------------------------
# Add the "mazehelper" function to the Magic Options
#-----------------------------------------------------------------

set Opts(mazerouter) 0

foreach window [magic::windownames layout] {

   set framename [winfo parent $window]
   if {$framename == "."} {
      set framename $window
   }
   set m ${framename}.titlebar.optsbutton.optsmenu
   $m add check -label "Maze Router" -variable Opts(mazerouter) -command \
	"if {$Opts(mazerouter) == 0} {destroy .mazehelper} else {genmazehelper}"
}


